#pragma once
#include <string>
#include <fstream>
#include <iostream>
#include <map>
#include <iterator>
#include <tuple>
#include <iomanip>
struct Record
{
    unsigned int id;
    char name[20];
    char typeWork[20];
    char units[5];
    double cost;
    char date[10];
    int scopeOfWork;

    friend std::istream& operator >> (std::istream& os, Record& _rec);
    friend std::ostream& operator << (std::ostream& os, const Record& _rec);
};

inline std::istream& operator >> (std::istream& os, Record& _rec)
{
    std::cout << "Enter id of record" << std::endl;
    os >> _rec.id;
    std::cout << "Enter company name" << std::endl;
    os >> _rec.name;
    std::cout << "Enter the type of work" << std::endl;
    os >> _rec.typeWork;
    std::cout << "Enter the units" << std::endl;
    os >> _rec.units;
    std::cout << "Enter the cost per unit of work" << std::endl;
    os >> _rec.cost;
    std::cout << "Enter date" << std::endl;
    os >> _rec.date;
    std::cout << "Enter the scope of work" << std::endl;
    os >> _rec.scopeOfWork;

    return os;
}

inline void addRecord(const std::string& fileName)
{
    Record temp;
    std::ofstream file(fileName, std::ios::binary | std::ios::app);
    std::cin >> temp;
    file.write(reinterpret_cast<char*>(&temp), sizeof(Record));

    file.close();
}

inline void deleteRecord(const std::string& fileName, const unsigned int& delId )
{
    std::ifstream file(fileName, std::ios::binary | std::ios::in);

    std::map<unsigned int, Record> records_map;

    Record temp;

    while (file.read(reinterpret_cast<char*>(&temp), sizeof(Record)), !file.eof())
    {
        if (temp.id != delId)
            records_map.insert(std::make_pair(temp.id,temp));
    }

    std::ofstream newFile("tempNew.dat", std::ios::binary | std::ios::out);

    for (auto& el : records_map)
    {
        newFile.write(reinterpret_cast<char*>(&el.second), sizeof(Record));
    }

    file.close();
    newFile.close();

    remove(fileName.c_str());
    rename("tempNew.dat", fileName.c_str());
}

inline void replaceRecord(const std::string& fileName, const Record& newRec)
{
    std::ifstream file(fileName, std::ios::binary | std::ios::in);

    std::map<unsigned int, Record> records_map;

    Record temp;

    while (file.read(reinterpret_cast<char*>(&temp), sizeof(Record)), !file.eof())
    {
        if (temp.id != newRec.id)
            records_map.insert(std::make_pair(temp.id, temp));
        else
            records_map.insert(std::make_pair(newRec.id, newRec));
    }

    std::ofstream newFile("tempNew.dat", std::ios::binary | std::ios::out);

    for (auto& el : records_map)
    {
        newFile.write(reinterpret_cast<char*>(&el.second), sizeof(Record));
    }

    file.close();
    newFile.close();

    remove(fileName.c_str());
    rename("tempNew.dat", fileName.c_str());
}

inline std::ostream& operator << (std::ostream& os, const Record& _rec)
{
    os << std::setw(3) << _rec.id << " | "
        << std::setw(10) << _rec.name << " | "
        << std::setw(15) << _rec.typeWork << " | "
        << std::setw(5) << _rec.units << " | "
        << std::setw(5) << _rec.cost << " | "
        << std::setw(10) << _rec.date << " | "
        << std::setw(5) << _rec.scopeOfWork
        << std::endl;

    return os;
}

 inline std::string fromBinToTxt(const std::string& fileName)
{
    std::ifstream file(fileName, std::ios::binary | std::ios::in);
    
    std::string newFileName = fileName;
    newFileName.resize(newFileName.size() - 4);
    newFileName += "_TXT.txt";
    Record temp;
    double sum = 0;
    std::ofstream newFile(newFileName);
    newFile << std::setw(3) << "id" << " | "
        << std::setw(10) << "firm" << " | "
        << std::setw(15) << "type of work" << " | "
        << std::setw(5) << "units" << " | "
        << std::setw(5) << "cost" << " | "
        << std::setw(10) << "date" << " | "
        << std::setw(5) << "scope"
        << std::endl;
    
    while (file.read(reinterpret_cast<char*>(&temp), sizeof(Record)), !file.eof())
    {
        sum += temp.cost * temp.scopeOfWork;
        newFile << temp;
    }

    newFile << "_______________________________________________________________________" << std::endl;
    newFile << std::left << std::setw(50) << "TOTAL:" << std::setw(21) <<std::right << sum;
    
    file.close();
    newFile.close();

    return newFileName;
}

inline std::string searchByFirmName(const std::string& fileName, const std::string& firmName)
{
    std::ifstream file(fileName, std::ios::binary | std::ios::in);
    std::ofstream fileResult("Result1_" + fileName , std::ios::binary | std::ios::out);

    Record temp;

    while (file.read(reinterpret_cast<char*>(&temp), sizeof(Record)), !file.eof())
    {
        if (temp.name == firmName)
            fileResult.write(reinterpret_cast<char*>(&temp), sizeof(Record));
    }

    file.close();
    fileResult.close();

    return ("Result1_" + fileName);
}

inline std::string searchByTypeOfWork(const std::string& fileName, const std::string& typeOfWork)
{
    std::ifstream file(fileName, std::ios::binary | std::ios::in);
    std::ofstream fileResult("Result2_" + fileName, std::ios::binary | std::ios::out);

    Record temp;

    while (file.read(reinterpret_cast<char*>(&temp), sizeof(Record)), !file.eof())
    {
        if (temp.typeWork == typeOfWork)
            fileResult.write(reinterpret_cast<char*>(&temp), sizeof(Record));
    }

    file.close();
    fileResult.close();

    return ("Result2_" + fileName);
}

inline std::string searchByDate(const std::string& fileName, const std::string& date)
{
    std::ifstream file(fileName, std::ios::binary | std::ios::in);
    std::ofstream fileResult("Result3_" + fileName, std::ios::binary | std::ios::out);

    Record temp;

    while (file.read(reinterpret_cast<char*>(&temp), sizeof(Record)), !file.eof())
    {
        if (temp.date == date)
            fileResult.write(reinterpret_cast<char*>(&temp), sizeof(Record));
    }

    file.close();
    fileResult.close();

    return ("Result3_" + fileName);
}

